<?php
/**
 * Translator Revolution WP Plugin
 * http://goo.gl/4E7Mx
 *
 * LICENSE
 *
 * You need to buy a license if you want use this script.
 * http://codecanyon.net/legal/market
 *
 * @package    Translator Revolution WP Plugin
 * @copyright  Copyright (c) 2015, SurStudio, www.surstudio.net
 * @license    http://codecanyon.net/licenses/standard
 * @version    1.8
 * @date       2015-02-02
 */

class SurStudioPluginTranslatorRevolutionLiteAdmin {

	public static function initialize() {

		if (!is_admin())
			return;

		add_action('admin_init', array('SurStudioPluginTranslatorRevolutionLiteAdmin', '_initialize'));
		add_action('admin_head', array('SurStudioPluginTranslatorRevolutionLiteAdmin', '_remove_messages'));
		add_action('plugins_loaded', array('SurStudioPluginTranslatorRevolutionLiteAdmin', 'initializeMenus'));

		add_action('wp_ajax_nopriv_surstudio_plugin_translator_revolution_lite_get_languages_per_resource', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_get_languages_per_resource'));
		add_action('wp_ajax_surstudio_plugin_translator_revolution_lite_get_languages_per_resource', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_get_languages_per_resource'));

		add_action('wp_ajax_nopriv_surstudio_plugin_translator_revolution_lite_get_cached_translations', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_get_cached_translations'));
		add_action('wp_ajax_surstudio_plugin_translator_revolution_lite_get_cached_translations', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_get_cached_translations'));

		add_action('wp_ajax_nopriv_surstudio_plugin_translator_revolution_lite_save_cached_translations', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_save_cached_translations'));
		add_action('wp_ajax_surstudio_plugin_translator_revolution_lite_save_cached_translations', array('SurStudioPluginTranslatorRevolutionLiteAdminTranslations', '_save_cached_translations'));

	}
	
	public static function _initialize() {

		self::_watch_events();

		self::_select_language();
		
		self::_load_styles();
		self::_load_scripts();
		
	}
	
	protected static function _watch_events() {

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isGeneratingPoFile())
			self::_po();

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isExportingTranslations()) {
			self::_export_translations();
			die();
		}

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isSavingSettings())
			self::_save_settings();

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isResetingSettings())
			self::_reset_settings();

	}
	
	protected static function _po() {

		header('Content-Type: text/html; charset=utf-8'); 
		SurStudioPluginTranslatorRevolutionLiteMessage::genPo();
		die();
		
	}
	
	protected static function _save_settings() {

		SurStudioPluginTranslatorRevolutionLiteAdminForm::save();
		
	}
	
	protected static function _reset_settings() {
		
		SurStudioPluginTranslatorRevolutionLiteAdminForm::reset();
		
	}
	
	protected static function _export_translations() {
		
		SurStudioPluginTranslatorRevolutionLiteTransexportField::run();
		
	}
	
	protected static function _load_scripts() {

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isLoadingAdminPage()) {
			wp_enqueue_script('surstudio-plugin-translator-revolution-lite-admin-common', SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_JS .'/common.class.js', 'jquery-ui-core', SurStudioPluginTranslatorRevolutionLiteConfig::getVersion());
			wp_enqueue_script('surstudio-plugin-translator-revolution-lite-admin', SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_JS .'/admin.class.js', 'jquery-ui-core', SurStudioPluginTranslatorRevolutionLiteConfig::getVersion());
			wp_enqueue_script('jquery');
			wp_enqueue_script('jquery-ui-core');
			wp_enqueue_script('thickbox');
		}

	}
	
	protected static function _load_styles() {

		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isLoadingAdminPage() || strpos(SurStudioPluginTranslatorRevolutionLiteCommon::getAdminWidgetsUrl(), $_SERVER['REQUEST_URI']) !== false) {
			wp_enqueue_style('surstudio-plugin-translator-revolution-lite-admin', SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CSS .'/admin.css', false, SurStudioPluginTranslatorRevolutionLiteConfig::getVersion(), 'screen');
			wp_enqueue_style('thickbox');
		}

	}
	
	public static function _remove_messages() {
	
		if (SurStudioPluginTranslatorRevolutionLiteAdminEvents::isLoadingAdminPage() || strpos(SurStudioPluginTranslatorRevolutionLiteCommon::getAdminWidgetsUrl(), $_SERVER['REQUEST_URI']) !== false)	
			SurStudioPluginTranslatorRevolutionLiteCommon::renderCSS('.update-nag,div.updated,div.error{display:none}');
		
	}
	
	public static function initializeMenus() {

		add_action('admin_menu', array('SurStudioPluginTranslatorRevolutionLiteAdmin', '_add_options_page'));

	}

	public static function _add_options_page() {
		
		add_submenu_page('plugins.php', SurStudioPluginTranslatorRevolutionLiteConfig::getName(false, true), SurStudioPluginTranslatorRevolutionLiteConfig::getName(false, true), 'manage_options', SurStudioPluginTranslatorRevolutionLiteConfig::getAdminHandle(), array('SurStudioPluginTranslatorRevolutionLiteAdmin', '_render_main_form'));
		
	}

	protected static function _gen_meta_tag_rules_for_tabs() {
		
		$tabs = array(
			array('general', 'advanced', 'translations', 'support'),
			array('advanced_general', 'advanced_import_export'),
			array('translations_general', 'translations_import', 'translations_export')
		);

		$current_tabs = array(
			SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab', 'POST'),
			SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab_2', 'POST'),
			SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab_3', 'POST')
		);

		$result = self::_gen_meta_tag_rules_for_tabs_aux($tabs, $current_tabs);

		return $result;
		
	}
	
	protected static function _gen_meta_tag_rules_for_tabs_aux($_tabs, $_currents, $_level=0) {
		
		$result = array();

		if (!is_array($_tabs[0])) {

			$current = $_currents[$_level];

			if (SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($current))
				$current = $_tabs[0];

			for ($i=0; $i<count($_tabs); $i++)
				$result[] = array(
					'expression' => $_tabs[$i] == $current,
					'tag' => $_tabs[$i] . '.show'
				);

		}
		else 
			for ($j=0; $j<count($_tabs); $j++) 
				$result = array_merge($result, self::_gen_meta_tag_rules_for_tabs_aux($_tabs[$j], $_currents, $j));

		$result[] = array(
			'expression' => SurStudioPluginTranslatorRevolutionLiteConfig::isSupportTabEnabled(),
			'tag' => 'support.enabled'
		);

		return $result;		
		
	}
	
	public static function _render_main_form() {

		$form = new SurStudioPluginTranslatorRevolutionLiteAdminForm();

		echo $form->render(array(
			'type' => 'file',
			'content' => '/admin/main_form.tpl',
			'meta_tag_rules' => self::_gen_meta_tag_rules_for_tabs()
		));
	
	}

	protected static function _select_language() {

		load_plugin_textdomain(SurStudioPluginTranslatorRevolutionLiteConfig::getAdminLanguageDomain(), false, dirname(plugin_basename(__FILE__)) . '/../languages');

	}
	
}

class SurStudioPluginTranslatorRevolutionLiteAdminTranslations {
	
	public static function _get_languages_per_resource() {
		
		$resource = base64_decode(SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_resource', 'POST')); //false
		
		if ($resource !== false) {
		
			$languages = self::_scan_languages_per_resource($resource);
			
			$json = array(
				'resource' => $resource,
				'languages' => $languages
			);
			
		}
		else {
			$json = array(
				'resource' => '',
				'languages' => array()
			);			
		}
		
		echo SurStudioPluginTranslatorRevolutionLiteFastJSON::encode($json);
		exit;
		
	}

	public static function _get_cached_translations() {
		
		$file = base64_decode(SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_languages', 'POST'));
		self::_render_cached_translations_form($file);
		exit;

	}

	public static function _save_cached_translations() {
		
		$file = base64_decode(SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_languages', 'POST'));
		$translations = self::_get_new_translations();
		
		$global_file = preg_replace('/[a-zA-Z0-9]{32}/', 'global', $file);
		
		self::_update_translations($file, $translations);
		self::_update_translations($global_file, $translations);
		
		exit;
		
	}
	
	protected static function _update_translations($_file, $_translations) {
		
		$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($_file);
		
		if (!$contents)
			return '';

		$xml = new DOMDocument();
		$xml->preserveWhiteSpace = false;

		if (@!$xml->loadXML($contents))
			return false;

		$xml->formatOutput = true;

		$xpath = new DOMXPath($xml); 

		foreach ($_translations as $hash => $translation)
			self::_update_translation($xml, $xpath, $hash, $translation);

		SurStudioPluginTranslatorRevolutionLiteFileHandler::write($_file, $xml->saveXML());

	}
	
	protected static function _update_translation(&$_xml, &$_xpath, $_hash, $_translation) {

		$current_translation = $_xpath->query("/translations/word[@hash='$_hash']/translation")->item(0);
		
		if (is_null($current_translation))
			return false;
		
		if (SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($_translation)) {
			$word = $_xpath->query("/translations/word[@hash='$_hash']")->item(0);
			$_xml->firstChild->removeChild($word);
		}
		else
			$current_translation->firstChild->nodeValue = $_translation;

	}
	
	protected static function _get_new_translations() {
		
		$result = array();
		
		$translations = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations', 'POST');

		if (!is_array($translations))
			return $result;
	
		foreach ($translations as $translation) {
			
			if (count($translation) != 2)
				continue;
				
			$result[$translation[0]] = $translation[1];
			
		}

		return $result;
		
	}

	
	public static function _render_cached_translations_form($_file) {
		
		$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($_file);
		
		if (!$contents)
			return '';

		$xml = new DOMDocument();
		$xml->preserveWhiteSpace = false;

		if (@!$xml->loadXML($contents))
			return '';

		$xpath = new DOMXPath($xml); 
		$scope = $xpath->query('/translations')->item(0)->getAttribute('scope');
		$from = $xpath->query('/translations')->item(0)->getAttribute('from');
		$to = $xpath->query('/translations')->item(0)->getAttribute('to');
		$from_language = SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($from);
		$to_language = SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($to);

		$cached_translations_fields_formatted = self::_get_cached_translations_fields_formatted($xml);

		$object = (object) array(
			'scope' => $scope,
			'from' => $from,
			'to' => $to,
			'from_language' => $from_language,
			'to_language' => $to_language,
			'cached_translations_fields_formatted' => $cached_translations_fields_formatted
		);

		$result = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($object, array(
			'type' => 'file',
			'content' => '/admin/translations_cached.tpl'
		));

		echo $result;

	}
	
	protected static function _get_cached_translations_fields_formatted($_xml) {
		
		$words = self::_get_cached_translation_words($_xml);
		
		$result = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($words, array(
			'type' => 'file',
			'content' => '/admin/translations_cached_field.tpl'
		));
		
		return $result;
		
	}
	
	protected static function _get_cached_translation_words($_xml) {

		$result = array();
		
		$xpath = new DOMXPath($_xml); 
		$words = $xpath->query('/translations/word');
		$sources = $xpath->query('/translations/word/source');
		$translations = $xpath->query('/translations/word/translation');

		for ($i=0; $i<$words->length; $i++) {
			$result[] = array(
				'hash' => $words->item($i)->getAttribute('hash'),
				'date' => $words->item($i)->getAttribute('date'),
				'source' => SurStudioPluginTranslatorRevolutionLiteCommon::escapeHtmlBrackets($sources->item($i)->nodeValue),
				'translation' => SurStudioPluginTranslatorRevolutionLiteCommon::escapeHtmlBrackets($translations->item($i)->nodeValue),
				'id' => 'surstudio_translations_cached'
			);
		}
		
		return $result;
		
	}
	
	protected static function _scan_languages_per_resource($_scope) {

		$files = SurStudioPluginTranslatorRevolutionLiteTranslationsField::getFilesPaths(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);
		$result = array();
		
		foreach ($files as $file) {
			
			$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($file);
			
			if (!$contents)
				continue;

			$xml = new DOMDocument();
			$xml->preserveWhiteSpace = false;
			
			if (@!$xml->loadXML($contents))
				continue;
			
			$xpath = new DOMXPath($xml); 
			$scope = $xpath->query('/translations')->item(0)->getAttribute('scope');
			
			if ($scope != $_scope)
				continue;
				
			$from = $xpath->query('/translations')->item(0)->getAttribute('from');
			$to = $xpath->query('/translations')->item(0)->getAttribute('to');

			$from_language = SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($from);
			$to_language = SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($to);

			if (!$from_language || !$to_language)
				continue;

			$result[$from_language . ' > ' . $to_language] = array(
				'file' => base64_encode($file),
				'from' => $from,
				'to' => $to
			);

		}

		asort($result);

		return $result;
		
	}
	
}

class SurStudioPluginTranslatorRevolutionLiteAdminBaseForm extends SurStudioPluginTranslatorRevolutionLiteItem {
	
	public $title_message;
	public $preview_message;
	public $activated_message;
	public $saved_message;
	public $reseted_message;
	public $save_button_message;
	public $reset_button_message;
	
	protected $_fields;
	
	public function __construct() {
		
		$this->title_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_title', SurStudioPluginTranslatorRevolutionLiteConfig::getName());

		$this->saved_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_saved');
		$this->reseted_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_reseted');
		$this->reset_message = SurStudioPluginTranslatorRevolutionLiteCommon::jsonCompatible(SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_reset_confirm'));
		$this->save_button_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_save_button');
		$this->reset_button_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_reset_button');

		$this->cached_translations_saved_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('translations_saved');
		
	}
	
	protected static function _commit($_name, $_result) {

		if (!get_option($_name))
			add_option($_name, $_result);
		else
			update_option($_name, $_result);
		
		if (!get_option($_name)) {
			delete_option($_name);
			add_option($_name, $_result);
		}
		
	}
	
	public function render($_options, $_html_encode=false) {
		
		return parent::render($_options, $_html_encode);
		
	}
	
	protected function _prepare_settings() {}
	protected function _set_fields() {}
	
}

class SurStudioPluginTranslatorRevolutionLiteAdminForm extends SurStudioPluginTranslatorRevolutionLiteAdminBaseForm {
	
	public $group_1;
	public $group_2;
	public $group_3;
	public $group_4;
	public $group_5;
	public $group_6;
	public $group_7;
	
	public $tab;
	public $tab_2;
	public $tab_3;
	
	public $general_message;
	public $general_api_key_validate_message;
	public $general_cache_folder_validate_message;
	public $general_cache_file_validate_message;
	public $advanced_message;
	public $advanced_general_message;
	public $advanced_import_export_message;
	public $translations_message;
	public $translations_general_message;
	public $translations_import_message;
	public $translations_export_message;
	public $support_message;
	
	public $ajax_url;
	
	protected static $_imported_status;
	
	public function __construct() {
		
		parent::__construct();
		
		$this->general_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('general_tab');
		$this->general_api_key_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_api_key_validate');
		$this->general_cache_folder_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_cache_folder_validate', SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);
		$this->general_openssl_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_openssl_validate');

		$this->general_api_key_empty_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_api_key_empty_validate');
		$this->general_api_key_google_empty_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_api_key_google_empty_validate');
		$this->general_client_id_microsoft_empty_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_client_id_microsoft_empty_validate');
		$this->general_client_secret_microsoft_empty_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_client_secret_microsoft_empty_validate');

		$this->advanced_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('advanced_tab');
		$this->advanced_general_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('advanced_general_tab');
		$this->advanced_import_export_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('advanced_import_export_tab');
		$this->advanced_import_success_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_imported_success');
		$this->advanced_import_fail_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_imported_fail');
		
		$this->translations_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('translations_tab');
		$this->translations_general_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('translations_general_tab');
		$this->translations_import_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('translations_import_tab');
		$this->translations_export_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('translations_export_tab');
		
		$this->support_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('support_tab');

		$this->ajax_url = SurStudioPluginTranslatorRevolutionLiteCommon::getAjaxUrl();

		$this->_set_fields();

	}
	
	protected static function _get_imported_status() {
		
		return self::$_imported_status;
		
	}
	
	protected static function _set_imported_status($_status) {
	
		self::$_imported_status = $_status;
		
	}
	
	protected static function _import_translations() {
		
		if (!SurStudioPluginTranslatorRevolutionLiteAdminEvents::isImportingTranslations())
			return null;
		
		return true;
		
	}
	
	protected static function _import() {
		
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getDefaults();
		$key = $settings['import']['id'];
		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($key, 'POST');
		
		if ($value === false || SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($value))
			return null;
		
		$decode = base64_decode($value);
		
		if ($decode === false) {
			self::_set_imported_status(false);
			return false;
		}
		
		$unserialize = @unserialize($decode);

		if (!is_array($unserialize)) {
			self::_set_imported_status(false);
			return false;
		}
		
		$result = array();

		foreach ($settings as $key => $setting) {
			
			if (in_array($key, array('import', 'export')))
				continue;
			
			if (array_key_exists($key, $unserialize))
				$result[$key] = $unserialize[$key];

		}

		if (count($result) == 0) {
			self::_set_imported_status(false);
			return false;
		}

		self::_commit(SurStudioPluginTranslatorRevolutionLiteConfig::getDbSettingsName(), $result);		
		self::_set_imported_status(true);
		
		return true;
		
	}
	
	public static function save() {
		
		if (!is_null(self::_import()))
			return;

		if (!is_null(self::_import_translations()))
			return;
		
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getDefaults();
		$result = array();

		foreach ($settings as $key => $setting) {
			
			$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($setting['id'], 'POST');
			
			switch ($key) {
				case 'languages': {
					$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable(str_replace('languages', 'languages_order', $setting['id']), 'POST');
					
					if ($value !== false) {
						$value = explode(',', $value);
						if ($value !== $setting['value'])
							$result[$key] = array('value' => $value);
						else
							unset($result[$key]);
					}
					else
						unset($result[$key]);
					
					break;
				}
				case 'translations_import':
				case 'translations_export':
				case 'import':
				case 'export': {
					continue;
					break;
				}
				default: {

					if ($key == 'id' || (SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($key, '_class') && $key != 'language_selector_class'))
						$value = trim(SurStudioPluginTranslatorRevolutionLiteCommon::cleanId($value));

					$unset_template = SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($key, '_template') && SurStudioPluginTranslatorRevolutionLiteCommon::stripBreakLinesAndTabs($value) == SurStudioPluginTranslatorRevolutionLiteCommon::stripBreakLinesAndTabs($setting['value']);

					if (!$unset_template && $value !== false && $value != $setting['value'])
						$result[$key] = array('value' => $value);
					else
						unset($result[$key]);
					break;

				}
			}
		}
		
		self::_commit(SurStudioPluginTranslatorRevolutionLiteConfig::getDbSettingsName(), $result);

	}
	
	public static function reset() {
		
		if (get_option(SurStudioPluginTranslatorRevolutionLiteConfig::getDbSettingsName()))
			delete_option(SurStudioPluginTranslatorRevolutionLiteConfig::getDbSettingsName());

	}

	protected function _google_validate() {
		
		return $this->_fields['translation_service']->value != 'gt' || SurStudioPluginTranslatorRevolutionLiteCommon::isOpenSSLInstalled();
		
	}
	
	protected function _cache_file_validate() {
		
		if (!$this->_cache_folder_validate())
			return true;
		
		$files_validate = SurStudioPluginTranslatorRevolutionLiteCommon::areFolderFilesWritable(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);
		
		if ($files_validate !== true) {
			$this->general_cache_file_validate_message = SurStudioPluginTranslatorRevolutionLiteMessage::get('plugin_cache_file_validate', $files_validate);
			return false;
		}
		
		return true;
		
	}
	
	protected function _cache_folder_validate() {
		
		return SurStudioPluginTranslatorRevolutionLiteCommon::isFolderWritable(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);
		
	}
	
	protected function _api_key_validate() {
		
		$value = $this->_fields['api_key']->value;
		
		if (empty($value))
			return true;
			
		if (strlen($value) != 56 && strlen($value) != 36)
			return false;
		
		return true;
		
	}
	
	protected function _client_secret_microsoft_empty_validate() {

		if ($this->_fields['translation_service']->value != 'ma')
			return true;
		
		return !empty($this->_fields['microsoft_client_secret']->value);
		
	}
	
	protected function _client_id_microsoft_empty_validate() {

		if ($this->_fields['translation_service']->value != 'ma')
			return true;
		
		return !empty($this->_fields['microsoft_client_id']->value);
		
	}
	
	protected function _api_key_google_empty_validate() {
		
		if ($this->_fields['translation_service']->value != 'gt')
			return true;
		
		return !empty($this->_fields['google_api_key']->value);
		
	}
	
	protected function _api_key_empty_validate() {
		
		return !empty($this->_fields['api_key']->value);
		
	}
	
	public function render($_options, $_html_encode=false) {
		
		$cache_folder_validate = $this->_cache_folder_validate();
		$cache_file_validate = $this->_cache_file_validate();
		SurStudioPluginTranslatorRevolutionLiteConfig::setCacheFlag($cache_folder_validate && $cache_file_validate);

		$this->_prepare_settings();

		$is_importing = SurStudioPluginTranslatorRevolutionLiteAdminEvents::isSavingSettings() && SurStudioPluginTranslatorRevolutionLiteValidator::isBool(self::_get_imported_status());
		$is_importing_translations = SurStudioPluginTranslatorRevolutionLiteAdminEvents::isImportingTranslations();

		if (!array_key_exists('meta_tag_rules', $_options))
			$_options['meta_tag_rules'] = array();

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_api_key_validate(),
			'tag' => 'api_key_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_api_key_empty_validate(),
			'tag' => 'api_key_empty_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_api_key_google_empty_validate(),
			'tag' => 'api_key_google_empty_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_client_id_microsoft_empty_validate(),
			'tag' => 'client_id_microsoft_empty_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_client_secret_microsoft_empty_validate(),
			'tag' => 'client_secret_microsoft_empty_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_cache_folder_validate(),
			'tag' => 'cache_folder_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $cache_file_validate,
			'tag' => 'cache_file_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $this->_google_validate(),
			'tag' => 'openssl_validate'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => SurStudioPluginTranslatorRevolutionLiteAdminEvents::isSavingSettings() && !$is_importing && !$is_importing_translations,
			'tag' => 'just_saved'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $is_importing_translations,
			'tag' => 'just_imported_translations_success'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $is_importing && self::_get_imported_status(),
			'tag' => 'just_imported_success'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => $is_importing && !self::_get_imported_status(),
			'tag' => 'just_imported_fail'
		);

		$_options['meta_tag_rules'][] = array(
			'expression' => SurStudioPluginTranslatorRevolutionLiteAdminEvents::isResetingSettings(),
			'tag' => 'just_reseted'
		);
		
		return parent::render($_options, $_html_encode);

	}

	protected function _set_fields() {
		
		if (is_array($this->_fields))
			return;
			
		$this->_fields = array();
			
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getSettings(true);
		
		foreach ($settings as $key => $setting) { 
			
			if (!array_key_exists('type', $setting))
				continue;
			
			$field_class = 'SurStudioPluginTranslatorRevolutionLite' . ucfirst($setting['type']) . 'Field';
			
			if ($field_class == 'SurStudioPluginTranslatorRevolutionLiteField')
				continue;
			
			$this->_fields[$key] = new $field_class($setting);
		}
		
	}
	
	protected function _prepare_settings() {
		
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getSettings();
		
		$groups = 7;
		
		for ($i=1; $i<$groups+1; $i++) {

			$partial = array();
			
			foreach ($this->_fields as $key => $field) {
				if ($field->group == $i) {
					$field->satisfyDependence($this->_fields);
					$partial[] = $field->output();
				}
			}

			$group = 'group_' . $i;
					
			$this->{$group} = implode("\n", $partial);
				
		}
		
		$tab = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab', 'POST');
		$this->tab = $tab !== false ? $tab : '';

		$tab_2 = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab_2', 'POST');
		$this->tab_2 = $tab_2 !== false ? $tab_2 : '';

		$tab_3 = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_tab_3', 'POST');
		$this->tab_3 = $tab_3 !== false ? $tab_3 : '';
		
	}
	
}

class SurStudioPluginTranslatorRevolutionLiteAdminEvents {

	public static function isLoadingAdminPage() {
		
		return in_array(SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('page', 'GET'), array(SurStudioPluginTranslatorRevolutionLiteConfig::getAdminHandle()));
		
	}
	
	public static function isSavingSettings() {
		
		return SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_plugin_translator_revolution_lite_admin_action', 'POST') === 'surstudio_plugin_translator_revolution_lite_save_settings';
		
	}
	
	public static function isResetingSettings() {
		
		return SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_plugin_translator_revolution_lite_admin_action', 'POST') === 'surstudio_plugin_translator_revolution_lite_reset_settings';
		
	}

	public static function isExportingTranslations() {

		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_export_button', 'POST');
		return self::isSavingSettings() && !empty($value);
		
	}
	
	public static function isImportingTranslations() {
		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_import_button', 'POST');
		return self::isSavingSettings() && !empty($value);
		
	}

	public static function isGeneratingPoFile() {
		
		return self::isLoadingAdminPage() && SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio-po', 'GET') !== false;
		
	}
	
}

SurStudioPluginTranslatorRevolutionLiteAdmin::initialize();

?>