<?php
/**
 * Translator Revolution WP Plugin
 * http://goo.gl/4E7Mx
 *
 * LICENSE
 *
 * You need to buy a license if you want use this script.
 * http://codecanyon.net/legal/market
 *
 * @package    Translator Revolution WP Plugin
 * @copyright  Copyright (c) 2015, SurStudio, www.surstudio.net
 * @license    http://codecanyon.net/licenses/standard
 * @version    1.8
 * @date       2015-02-02
 */

abstract class SurStudioPluginTranslatorRevolutionLiteItem {

	public $collection_item_index;
	protected $_properties;

	public function __construct($_properties=null) {

		if (is_object($_properties)) {
			$this->_properties = $_properties;
			$this->_set_properties();
		}

	}

	protected function _set_properties() {

		$this->setProperties($this->_properties);

	}

	public function getProperty($_property, $_html_entities=false) {

		return !$_html_entities ? $this->{$_property} : htmlentities($this->{$_property});

	}

	public function setProperties($_properties) {

		if (!is_null($_properties))
			foreach($_properties as $property => $value) 
				$this->setProperty($property, $value);

	}

	public function setProperty($_property, $_value) {

		return $this->{$_property} = $_value;

	}

	public function render($_options, $_html_encode=false) {

		if (array_key_exists('extra', $_options))
			if (array_key_exists('json', $_options['extra']))
				if ($_options['extra']['json'])
					$this->_json();

		if (array_key_exists('extra', $_options))
			if (array_key_exists('property', $_options['extra']))
				foreach ($_options['extra']['property'] as $property => $value)
					$this->{$property} = $value;

		$result = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($this, $_options, $_html_encode);

		return $result;

	}

	protected function _json() {

		// seems like there is some kind of bug in apache, so the field names have to be grabbed like this
		$fields = array();

		foreach ($this as $property => $value)
			if (!SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($property, '_json'))
				$fields[] = $property;

		foreach ($fields as $value)
			if (substr($value, 0, 1) != '_')
				$this->setProperty($value . '_json', SurStudioPluginTranslatorRevolutionLiteCommon::jsonCompatible($this->getProperty($value)));

	}

}

abstract class SurStudioPluginTranslatorRevolutionLiteField {

	public $id;
	public $option_id;
	public $value;
	public $dependence;
	public $dependence_show_value;

	public $title_message;
	public $description_message;

	public $dependence_count;

	public $formatted_dependence;
	public $formatted_dependence_show_value;

	protected $_dependence;

	public function __construct($_properties) {

		$this->_set_properties($_properties);

	}

	protected function _set_properties($_properties) {

		foreach ($_properties as $property => $value)
			$this->{$property} = $value;

	}

	public function satisfyDependence($_fields) {

		if (SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($this->dependence))
			return;

		$this->_dependence = SurStudioPluginTranslatorRevolutionLiteCommon::getArrayItems($this->dependence, $_fields);
		if (is_null($this->dependence_count))
			$this->dependence_count = count($this->_dependence);

	}

	protected function _has_dependence() {

		return !is_null($this->dependence) && !SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($this->dependence);

	}

	protected function _dependence_show() {

		if (!is_array($this->_dependence))
			return true;

		$result = array();

		if (is_array($this->dependence_show_value)) {
			if (count($this->dependence_show_value) == count($this->_dependence)) {
				$keys = array_keys($this->_dependence);
				for ($i=0; $i<count($keys); $i++) {
					$field = $this->_dependence[$keys[$i]];
					if ($field->value == $this->dependence_show_value[$i])
						$result[] = $field->id;
				}
				return count($result) == count($this->_dependence);
			}
		}

		foreach ($this->_dependence as $field)
			if (SurStudioPluginTranslatorRevolutionLiteCommon::inArray($field->value, $this->dependence_show_value))
				$result[] = $field->id;

		return count($result) == count($this->_dependence);


	}

	protected function _get_formatted_dependence() {

		$result = array();

		if (!$this->_has_dependence())
			return '';

		foreach ($this->_dependence as $field)
			$result[] = $field->id;

		return implode(',', $result);

	}

	protected function _render($_options, $_html_encode) {
		
		$this->formatted_dependence = is_array($this->dependence) ? implode(',', $this->dependence) : $this->dependence;
		$this->formatted_dependence_show_value = is_array($this->dependence_show_value) ? implode(',', $this->dependence_show_value) : $this->dependence_show_value;

		$options = $_options;

		if (!array_key_exists('meta_tag_rules', $options))
			$options['meta_tag_rules'] = array();

		$options['meta_tag_rules'][] = array(
			'expression' => $this->_has_dependence(),
			'tag' => 'has_dependence'
		);

		$options['meta_tag_rules'][] = array(		
			'expression' => $this->_dependence_show(),
			'tag' => 'dependence.show'
		);

		$result = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($this, $options, $_html_encode);

		return $result;
		
	}
	
	public function output($_html_encode=false) {
		
	}
	
	public function render($_options, $_html_encode=false) {

		return $this->_render($_options, $_html_encode);

	}

}

class SurStudioPluginTranslatorRevolutionLiteTextField extends SurStudioPluginTranslatorRevolutionLiteField {

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/text.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteFaqField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $q_1_message;
	public $a_1_message;
	public $q_2_message;
	public $a_2_message;
	public $q_3_message;
	public $a_3_message;
	public $q_4_message;
	public $a_4_message;
	public $q_5_message;
	public $a_5_message;

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/faq.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteSupportField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $step_1_message;
	public $step_2_message;
	public $step_3_message;
	public $step_4_message;

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/support.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteHeadingField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $group;

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/heading.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->value == "true",
					'tag' => 'value'
				),
				array(
					'expression' => !SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($this->description_message),
					'tag' => 'description'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteToggleField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $name;
	public $value_true;
	public $option_true;
	public $value_false;
	public $option_false;

	protected function _set_properties($_properties) {

		foreach ($_properties as $property => $value)
			$this->{$property} = $value;

		$this->name = $this->id;
		$keys = array_keys($_properties['values']);
		$this->value_true = $keys[0];
		$this->option_true = $_properties['values'][$keys[0]];
		$this->value_false = $keys[1];
		$this->option_false = $_properties['values'][$keys[1]];

	}

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/toggle.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->value == $this->value_true,
					'tag' => 'value_true.checked'
				),
				array(
					'expression' => $this->value == $this->value_false,
					'tag' => 'value_false.checked'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteRangeField extends SurStudioPluginTranslatorRevolutionLiteField {

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/range.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteTextareaField extends SurStudioPluginTranslatorRevolutionLiteField {

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/textarea.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteExportField extends SurStudioPluginTranslatorRevolutionLiteField {

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/export.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteLanguageOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $base_url;
	public $value_formatted;

	public function __construct($_properties) {

		parent::__construct($_properties);
		
		$this->base_url = SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_IMAGES . '/';
		$this->value_formatted = preg_replace('/[^a-zA-Z0-9]+|\s+/', '_', $this->value);

	}

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/language_option.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->checked,
					'tag' => 'checked'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteLanguageOrderOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $base_url;
	public $value_formatted;

	public function __construct($_properties) {

		parent::__construct($_properties);
		
		$this->base_url = SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_IMAGES . '/';
		$this->value_formatted = preg_replace('/[^a-zA-Z0-9]+|\s+/', '_', $this->value);

	}

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/language_order_option.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteLanguageField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $title_order_message;
	public $description_order_message;
	
	public $collection_formatted;
	public $collection_order_formatted;
	
	public $value_order;
	
	public $columns;
	protected $_collections;
	protected $_collection_order;

	public function __construct($_properties) {

		$this->_set_properties($_properties);
		$this->_set_options();
		$this->_set_order_options();

	}

	protected function _set_options() {

		$this->_collections = array();
		for ($i=0; $i<$this->columns; $i++)
			$this->_collections[$i] = new SurStudioPluginTranslatorRevolutionLiteItemCollection();

		$count = 0;
		foreach ($this->values as $key => $value) {

			$group = $count % $this->columns;

			$this->_collections[$group]->add(new SurStudioPluginTranslatorRevolutionLiteLanguageOptionField((object) array(
				'id' => SurStudioPluginTranslatorRevolutionLiteCommon::cleanId($this->id . '_' . $key, '_'),
				'name' => $this->id,
				'checked' => is_array($this->value) ? in_array((string) $key, $this->value, true) : false,
				'option' => $key,
				'value' => $value
			)), $key);

			$count++;

		}

	}

	protected function _set_order_options() {
		
		$this->value_order = is_array($this->value) ? implode(',', $this->value) : $this->value;
		
		$this->_collection_order = new SurStudioPluginTranslatorRevolutionLiteItemCollection();
		
		$values = is_array($this->value) ? $this->value : array($this->value);

		$items = array();
		
		foreach ($this->values as $key => $value) {
			
			if (!in_array((string) $key, $values, true))
				continue;
			
			$items[array_search($key, $values)] = new SurStudioPluginTranslatorRevolutionLiteLanguageOrderOptionField((object) array(
				'id' => SurStudioPluginTranslatorRevolutionLiteCommon::cleanId($this->id . '_order_' . $key, '_'),
				'option' => $key,
				'value' => $value
			));
			
		}
		
		ksort($items);
		
		foreach ($items as $key => $item)
			$this->_collection_order->add($item, $key);
		
	}

	public function output($_html_encode=false) {

		$this->collection_formatted = '';
		
		foreach ($this->_collections as $collection)
			$this->collection_formatted .= $collection->render(array(
				'type' => 'file',
				'content' => '/admin/language_option_group.tpl'
			), $_html_encode);

		$this->collection_order_formatted = $this->_collection_order->render(array(
			'type' => 'file',
			'content' => '/admin/language_order_option_group.tpl'
		), $_html_encode);

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/language.tpl'
		), $_html_encode);

		return $result;

	}	
	
}

class SurStudioPluginTranslatorRevolutionLiteCheckboxOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/Checkbox_option.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->checked,
					'tag' => 'checked'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteCheckboxField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $collection_formatted;
	protected $collection;

	public function __construct($_properties) {

		$this->_set_properties($_properties);
		$this->_set_options();

	}

	protected function _set_options() {

		$this->collection = new SurStudioPluginTranslatorRevolutionLiteItemCollection();

		foreach ($this->values as $key => $value) {

			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteCheckboxOptionField((object) array(
				'id' => SurStudioPluginTranslatorRevolutionLiteCommon::cleanId($this->id . '_' . $key, '_'),
				'name' => $this->id,
				'checked' => is_array($this->value) ? in_array((string) $key, $this->value, true) : false,
				'option' => $key,
				'value' => $value
			)), $key);

		}

	}

	public function output($_html_encode=false) {

		$this->collection_formatted = $this->collection->render(array(
			'type' => 'html',
			'content' => '{{ collection }}'
		), $_html_encode);

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/checkbox.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteExcategoryField extends SurStudioPluginTranslatorRevolutionLiteExclitemField {

	public function __construct($_properties) {
		
		parent::__construct($_properties, 'category');
		
	}
	
}

class SurStudioPluginTranslatorRevolutionLiteExpageField extends SurStudioPluginTranslatorRevolutionLiteExclitemField {

	public function __construct($_properties) {
		
		parent::__construct($_properties, 'page');
		
	}
	
}

class SurStudioPluginTranslatorRevolutionLiteExpostField extends SurStudioPluginTranslatorRevolutionLiteExclitemField {
	
	public function __construct($_properties) {
		
		parent::__construct($_properties, 'post');
	
	}
	
}

class SurStudioPluginTranslatorRevolutionLiteExclitemOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $indent;

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/exclude_item_option.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->checked,
					'tag' => 'checked'
				),
				array(
					'expression' => $this->indent != 0,
					'tag' => 'indent'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteExclitemField extends SurStudioPluginTranslatorRevolutionLiteCheckboxField {
	
	public $values;
	protected $_items;
	protected $_kind;
	
	public function __construct($_properties, $_kind) {

		$this->_kind = $_kind;

		$this->_set_properties($_properties);
		$this->_gen_values();
		$this->_set_options();

	}
	
	protected function _gen_items() {
		
		$this->_items = array();
		
		switch ($this->_kind) {
			case 'page': {
				$temp = get_pages();
				break;
			}
			case 'post': {
				$temp = get_posts();
				break;
			}
			case 'category': {
				$temp = get_categories(array(
					'hide_empty' => 0
				));
				break;
			}
			default: {
				return;
				break;
			}
		}			

		switch ($this->_kind) {
			case 'post':
			case 'page': {
				foreach ($temp as $item)
					$this->_items[$item->ID] = array(
						'title' => $item->post_title,
						'parent' => $item->post_parent
					);
				break;
			}
			case 'category': {
				foreach ($temp as $item)
					$this->_items[$item->cat_ID] = array(
						'title' => $item->cat_name,
						'parent' => $item->category_parent
					);
				break;
			}
		}
		
	}
	
	protected function _gen_values() {
	
		$this->_gen_items();
		$this->values = array();
		
		foreach ($this->_items as $id => $item)
			$this->values[$id] = array(
				'value' => $item['title'],
				'id' => $id,
				'parent' => $item['parent'],
			);

		$this->_sort();

	}
	
	protected function _sort() {
	
		if (count($this->values) < 1)
			return;
	
		$result = array();
		$temp = SurStudioPluginTranslatorRevolutionLiteCommon::chain('id', 'parent', 'value', $this->values);

		if (is_array($temp))		
			foreach ($temp as $item)
				$result[$item['id']] = $item;
		
		$this->values = $result;
		
	}
	
	protected function _set_options() {

		$this->collection = new SurStudioPluginTranslatorRevolutionLiteItemCollection();

		foreach ($this->values as $key => $value) {

			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteExclitemOptionField((object) array(
				'id' => SurStudioPluginTranslatorRevolutionLiteCommon::cleanId($this->id . '_' . $key, '_'),
				'name' => $this->id,
				'checked' => is_array($this->value) ? in_array((string) $key, $this->value, true) : false,
				'option' => $key,
				'indent' => $value['indent'] * 20,
				'value' => $value['value']
			)), $key);

		}

	}
	
	public function output($_html_encode=false) {

		$this->collection_formatted = $this->collection->render(array(
			'type' => 'html',
			'content' => '{{ collection }}'
		), $_html_encode);

		$result = $this->_render(array(
			'type' => 'file',
			'content' => '/admin/exclude_item.tpl'
		), $_html_encode);

		return $result;

	}	
	
}

class SurStudioPluginTranslatorRevolutionLiteRadioOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $indent;

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/radio_option.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->checked,
					'tag' => 'checked'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteRadioField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $collection_formatted;
	protected $collection;

	public function __construct($_properties) {

		$this->_set_properties($_properties);
		$this->_set_options();

	}

	protected function _set_options() {

		$this->collection = new SurStudioPluginTranslatorRevolutionLiteItemCollection();

		foreach ($this->values as $key => $value) {

			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteRadioOptionField((object) array(
				'id' => $this->id . '_' . $key,
				'name' => $this->id,
				'checked' => $this->value == $key,
				'option' => $key,
				'value' => $value
			)), $key);

		}

	}

	public function output($_html_encode=false) {

		$this->collection_formatted = $this->collection->render(array(
			'type' => 'html',
			'content' => '{{ collection }}'
		), $_html_encode);

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/radio.tpl'
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteSelectOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	protected $_parent;

	public function __construct($_properties, $_parent) {

		$this->_set_properties($_properties);
		$this->_parent = $_parent;

	}

	public function output($_html_encode=false) {

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/select_option.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => $this->selected,
					'tag' => 'selected'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteSelectField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $force_selected;
	
	public $collection_formatted;
	protected $collection;

	public function __construct($_properties) {

		$this->_set_properties($_properties);
		$this->_set_options();

	}

	protected function _set_options() {

		$this->collection = new SurStudioPluginTranslatorRevolutionLiteItemCollection();
		$selected_flag = false;

		foreach ($this->values as $key => $value) {

			if ($this->value == $key)
				$selected_flag = true;

			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteSelectOptionField((object) array(
				'selected' => $this->value == $key,
				'option' => $key,
				'value' => $value
			), $this), $key);

		}

		if ($this->force_selected === true && $selected_flag !== true)
			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteSelectOptionField((object) array(
				'selected' => true,
				'option' => $this->value,
				'value' => $this->value
			), $this), $this->value);

	}

	public function output($_template='select.tpl', $_html_encode=false) {

		$this->collection_formatted = $this->collection->render(array(
			'type' => 'html',
			'content' => '{{ collection }}'
		), $_html_encode);

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/' . $_template,
			'meta_tag_rules' => array(
				array(
					'expression' => property_exists($this, 'post_id') && !SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($this->post_id),
					'tag' => 'has_post_id'
				)
			)
		), $_html_encode);

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteTranslationsField extends SurStudioPluginTranslatorRevolutionLiteField {
	
	public $resource_formatted;
	
	protected $_resource_field;
	protected static $_files_scopes;
	
	protected $folder;
	
	public function __construct($_properties) {
		
		parent::__construct($_properties);
		$this->_initialize_resource_field($_properties);
		
	}
	
	protected function _initialize_resource_field($_properties) {

		$_resource_field = $_properties;
		$_resource_field['id'] = $_resource_field['id'] . '_resource';
		$_resource_field['values'] = $this->_get_resource_values();
		
		$this->_resource_field = new SurStudioPluginTranslatorRevolutionLiteSelectField($_resource_field);
		
	}
	
	public static function getFilesPaths($_folder, $_globals=false) {
		
		$contents = @scandir($_folder);
		$result = array();
		
		foreach ($contents as $name) {
			$file = $_folder . '/' . $name;
			if (SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($name, '.xml') && !SurStudioPluginTranslatorRevolutionLiteCommon::startsWith($name, 'log_') && @is_file($file) && ($_globals || !SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($name, 'global.xml')))
				$result[] = $file;
		}
		
		return $result;
		
	}
	
	protected function _get_files_scopes() {

		if (!is_null(self::$_files_scopes))
			return self::$_files_scopes;

		$files = self::getFilesPaths($this->folder);
		$result = array();
		
		foreach ($files as $file) {
			
			$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($file);
			
			if (!$contents)
				continue;

			$xml = new DOMDocument();
			$xml->preserveWhiteSpace = false;
			
			if (@!$xml->loadXML($contents))
				continue;
			
			$xpath = new DOMXPath($xml); 
			$scope = $xpath->query("/translations")->item(0)->getAttribute('scope');
			$result[base64_encode($scope)] = $scope;

		}

		asort($result);

		return self::$_files_scopes = $result;
		
	}
	
	protected function _get_resource_values() {
		
		$result = array_merge(array('' => ''), $this->_get_files_scopes());
		return $result;
		
	}
	
	public function output($_template='translations.tpl', $_html_encode=false) {

		$this->resource_formatted = $this->_resource_field->output('translations_select.tpl', $_html_encode);

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/' . $_template,
			'meta_tag_rules' => array(
				array(
					'expression' => false,
					'tag' => 'xxx'
				)
			)
		), $_html_encode);

		return $result;

	}
	
}

class SurStudioPluginTranslatorRevolutionLiteTransexportField extends SurStudioPluginTranslatorRevolutionLiteCheckboxField {

	public $button_message;
	public $empty_collection_message;
	
	public $global_message;

	public $values;
	public $resource_formatted;
	
	protected $folder;

	public function __construct($_properties) {
		
		$this->_set_properties($_properties);
		$this->_gen_values();
		$this->_set_options();	
	
	}
	
	protected function _gen_values() {
	
		$this->values = $this->_get_files();

	}
	
	public static function run() {

		if (!SurStudioPluginTranslatorRevolutionLiteAdminEvents::isExportingTranslations())
			return;
	
		$csv = new SurStudioPluginTranslatorRevolutionLiteParseCSV();
		
		$data = self::_get_data();
		
		header('Content-type: application/csv');
		header('Content-Disposition: attachment; filename="translations-' . date('c') . '.csv"');

		echo $csv->unparse($data);
		die();

	}
	
	protected static function _get_data() {
		
		$result = array();
		$inputs = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable('surstudio_translations_export', 'POST');
		
		foreach ($inputs as $single) {
			
			$file = SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE . '/' . @base64_decode($single) . '.xml';

			if (!is_file($file))
				continue;
		
			$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($file);
			
			if (!$contents)
				continue;

			$xml = new DOMDocument();
			$xml->preserveWhiteSpace = false;
			
			if (@!$xml->loadXML($contents))
				continue;

			$xpath = new DOMXPath($xml); 
			$words = $xpath->query("/translations/word");
			
			foreach ($words as $word) {
				
				$source = $word->getElementsByTagName('source');
				$translation = $word->getElementsByTagName('translation');
				
				if ($source->length != 1 || $translation->length != 1)
					continue;
				
				$result[] = array($source->item(0)->nodeValue, $translation->item(0)->nodeValue);
				
			}

		}

		return $result;
		
	}
	
	protected function _get_files() {

		$files = SurStudioPluginTranslatorRevolutionLiteTransimportField::getFilesPaths($this->folder, true);
		$result = array();
		
		foreach ($files as $file) {
			
			$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($file);
			
			if (!$contents)
				continue;

			$xml = new DOMDocument();
			$xml->preserveWhiteSpace = false;
			
			if (@!$xml->loadXML($contents))
				continue;

			$temp = explode('_', basename($file, ".xml"));
			$from = $temp[0];
			$to = $temp[1];
			$hash = $temp[2];
			
			$xpath = new DOMXPath($xml); 
			$scope = $xpath->query("/translations")->item(0)->getAttribute('scope');
			
			if (!array_key_exists($scope, $result))
				$result[$scope] = array(
					'hash' => $hash,
					'languages' => array()
				);
			
			$result[$scope]['languages'][] = array(
				'from' => $from,
				'to' => $to,
				'value' => SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($from) . ' &gt; ' . SurStudioPluginTranslatorRevolutionLiteCommon::getLanguage($to)
			);

		}

		ksort($result);

		return $result;
		
	}

	protected function _set_options() {

		$this->collection = new SurStudioPluginTranslatorRevolutionLiteItemCollection();

		$i = 0;
		foreach ($this->values as $scope => $value) {

			$this->collection->add(new SurStudioPluginTranslatorRevolutionLiteTransexportOptionField((object) array(
				'scope' => $scope == 'global' ? $this->global_message : $scope,
				'ix' => $i,
				'id' => $this->id,
				'values' => $value
			)), $scope);

			$i++;

		}

	}

	public function output($_html_encode=false) {

		$this->collection_formatted = $this->collection->render(array(
			'type' => 'html',
			'content' => '{{ collection }}'
		), $_html_encode);

		$result = $this->_render(array(
			'type' => 'file',
			'content' => '/admin/translations_export.tpl',
			'meta_tag_rules' => array(
				array(
					'expression' => empty($this->collection->collection),
					'tag' => 'empty_collection'
				)
			)
		), $_html_encode);

		return $result;

	}
	
}

class SurStudioPluginTranslatorRevolutionLiteTransexportOptionField extends SurStudioPluginTranslatorRevolutionLiteField {

	public $ix;
	public $scope;
	public $values;
	
	public function __construct($_properties) {
		
		parent::__construct($_properties);
		
	}

	public static function sort($_a, $_b) {
		
		$result = strcmp($_a['value'], $_b['value']);
		
		return $result;
		
	}
	
	public function output($_html_encode=false) {

		$result = '<div class="surstudio_plugin_translator_revolution_lite_transexport_item';

		if (!($this->ix % 3))
			$result .= ' surstudio_plugin_translator_revolution_lite_transexport_first_item';

		$result .= '">';

		$result .= SurStudioPluginTranslatorRevolutionLiteCommon::renderObject((object) array(
				'indent' => '',
				'name' => $this->id . '_foo',
				'id' => $this->id . '_' . $this->values['hash'],
				'value' => $this->scope,
				'option' => ''
			),			
			array(
				'type' => 'file',
				'content' => '/admin/translations_export_option.tpl',
				'meta_tag_rules' => array(
					array(
						'expression' => false,
						'tag' => 'indent'
					)
				)
			), $_html_encode);
		
		usort($this->values['languages'], array('SurStudioPluginTranslatorRevolutionLiteTransexportOptionField', 'sort'));
		
		foreach ($this->values['languages'] as $single) {

			$result .= SurStudioPluginTranslatorRevolutionLiteCommon::renderObject((object) array(
					'indent' => '20',
					'name' => $this->id,
					'id' => $this->id . '_' . $single['from'] . '_' . $single['to'] . '_' . $this->values['hash'],
					'value' => $single['value'],
					'option' => base64_encode($single['from'] . '_' . $single['to'] . '_' . $this->values['hash'])
				),			
				array(
					'type' => 'file',
					'content' => '/admin/translations_export_option.tpl',
					'meta_tag_rules' => array(
						array(
							'expression' => true,
							'tag' => 'indent'
						)
					)
				), $_html_encode);
			
		}
		
		$result .= '</div>';

		return $result;

	}	

}

class SurStudioPluginTranslatorRevolutionLiteTransimportField extends SurStudioPluginTranslatorRevolutionLiteField {
	
	public $upload_message;
	public $upload_button_message;
	public $from_message;
	public $to_message;
	public $resource_title_message;
	public $resource_description_message;
	public $resource_all_message;
	public $resource_global_message;
	public $action_title_message;
	public $action_description_message;
	public $action_add_and_edit;
	public $action_add;
	public $action_edit;
	public $test_title_message;
	public $test_description_message;
	public $test_true;
	public $test_false;
	
	public $button_message;
	
	public $validate_csv_empty_message;
	public $validate_csv_invalid_message;
	public $validate_language_empty_message;
	public $validate_resource_invalid_message;
	public $validate_action_invalid_message;
	public $validate_test_invalid_message;
	
	public $log_added_message;
	public $log_edited_message;
	public $log_removed_message;
	public $log_omitted_message;
	public $log_created_message;
	public $log_yes_created_message;
	public $log_no_created_message;
	
	public $log_test_message;

	public $no_import_message;

	public $log_title_message;
	public $csv_file_value_formatted;
	public $from_value_formatted;
	public $to_value_formatted;
	public $resource_value_formatted;
	public $action_value_formatted;
	public $test_value_formatted;

	public $from_formatted;
	public $to_formatted;
	public $resource_formatted;
	public $action_formatted;
	public $test_formatted;
	public $log_formatted;
	
	protected $_from_language_field;
	protected $_to_language_field;
	protected $_resource_field;
	protected $_action_field;
	protected $_test_field;
	
	protected $folder;
	protected $_files;
	
	protected $_errors;
	
	protected $_csv;
	protected $_log;
	
	protected static $_files_scopes;
	
	public function __construct($_properties) {
		
		parent::__construct($_properties);
		
		$this->_initialize_resource_field($_properties);
		$this->_initialize_language_fields($_properties);
		$this->_initialize_action_field($_properties);
		$this->_initialize_test_field($_properties);

		$this->_files = $_FILES;
		$this->_errors = array();
		$this->_log = array();

		$this->_run();
		
	}

	protected function _run() {
		
		if (!SurStudioPluginTranslatorRevolutionLiteAdminEvents::isImportingTranslations())
			return;

		if (!$this->_validate())
			return;

		$this->_import();

	}
	
	protected function _import() {
		
		$translations = array();
		
		foreach ($this->_csv as $single)
			$translations[] = array(
				'source' => $single[0],
				'translation' => $single[1],
				'hash' => SurStudioPluginTranslatorRevolutionLiteCommon::hashText($single[0])
			);

		$resources = array();
		if ($this->_resource_field->value == 'all' || $this->_resource_field->value == 'global') {
			
			$resources[] = 'global';
			
			if ($this->_resource_field->value == 'all') {
				$files_scopes = array_keys($this->_get_files_scopes());
				$files_scopes = array_filter($files_scopes);
				$resources = array_merge(
					$resources,
					$files_scopes
				);
			}
			
		}
		else
			$resources[] = $this->_resource_field->value;

		$this->_log = SurStudioPluginTranslatorRevolutionLiteTranslateTransport::importTranslations($translations, $resources, $this->_from_language_field->value, $this->_to_language_field->value, $this->_action_field->value, $this->_test_field->value == 'true');
		
	}
	
	protected function _validate() {
	
		if (!$this->_validate_csv_file())
			$this->_errors['csv_file'] = true;

		if (!$this->_validate_from())
			$this->_errors['from'] = true;

		if (!$this->_validate_to())
			$this->_errors['to'] = true;

		if (!$this->_validate_resource())
			$this->_errors['resource'] = true;
			
		if (!$this->_validate_action())
			$this->_errors['action'] = true;
			
		if (!$this->_validate_test())
			$this->_errors['test'] = true;
			
		return empty($this->_errors);
		
	}
	
	protected function _validate_test() {

		if (!array_key_exists($this->test_value_formatted, $this->_test_field->values)) {
			$this->test_value_formatted = $this->validate_test_invalid_message;
			return false;
		}
		
		return true;
		
	}

	protected function _validate_action() {

		if (!array_key_exists($this->action_value_formatted, $this->_action_field->values)) {
			$this->action_value_formatted = $this->validate_action_invalid_message;
			return false;
		}
		
		return true;
		
	}
	
	protected function _validate_resource() {
		
		if (!array_key_exists($this->resource_value_formatted, $this->_resource_field->values)) {
			$this->resource_value_formatted = $this->validate_resource_invalid_message;
			return false;
		}
		
		return true;
		
	}
	
	protected function _validate_from() {

		if (empty($this->from_value_formatted) || !array_key_exists($this->from_value_formatted, $this->_from_language_field->values)) {
			$this->from_value_formatted = $this->validate_language_empty_message;
			return false;
		}
		
		if ($this->_from_language_field->value == $this->_to_language_field->value) {
			$this->from_value_formatted = $this->_from_language_field->values[$this->from_value_formatted];
			return false;
		}
		
		return true;
		
	}
	
	protected function _validate_to() {

		if (empty($this->to_value_formatted) || !array_key_exists($this->to_value_formatted, $this->_to_language_field->values)) {
			$this->to_value_formatted = $this->validate_language_empty_message;
			return false;
		}

		if ($this->_to_language_field->value == $this->_from_language_field->value) {
			$this->to_value_formatted = $this->_to_language_field->values[$this->to_value_formatted];
			return false;
		}
		
		return true;
		
	}
	
	protected function _validate_csv_file() {

		if (empty($this->_files)) {
			$this->csv_file_value_formatted = $this->validate_csv_empty_message;
			return false;
		}
		
		if (!array_key_exists($this->id . '_csv', $this->_files)) {
			$this->csv_file_value_formatted = $this->validate_csv_empty_message;
			return false;
		}
		
		if ($this->_files[$this->id . '_csv']['error'] !== UPLOAD_ERR_OK) {
			$this->csv_file_value_formatted = SurStudioPluginTranslatorRevolutionLiteMessage::get('file_upload_error_' . $this->_files[$this->id . '_csv']['error']);
			return false;
		}
		
		$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($this->_files[$this->id . '_csv']['tmp_name']);
		if ($contents === false) {
			$this->csv_file_value_formatted = SurStudioPluginTranslatorRevolutionLiteMessage::get('file_upload_error_4');
			return false;			
		}

		$csv = new SurStudioPluginTranslatorRevolutionLiteParseCSV();
		$csv->parse($contents);
				
		$this->_csv = array();
		foreach ($csv->data as $single)
			if (count($single) == 2)
				$this->_csv[] = $single;
		
		if (empty($this->_csv)) {
			$this->csv_file_value_formatted = $this->validate_csv_invalid_message;
			return false;
		}
		
		$this->csv_file_value_formatted = $this->_files[$this->id . '_csv']['name'];
		
		return true;
		
	}
	
	protected function _initialize_test_field($_properties) {
		
		$field = $_properties;
		$field['title_message'] = $this->test_title_message;
		$field['description_message'] = $this->test_description_message;
		$field['id'] = $field['id'] . '_test';
		$field['values'] = array(
			'false' => $this->test_false,
			'true' => $this->test_true
		);
		$field['value'] = 'false';
		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($field['id'], 'POST');
		if ($value !== false)
			$this->test_value_formatted = $field['value'] = $value;
		
		$this->_test_field = new SurStudioPluginTranslatorRevolutionLiteToggleField($field);
		
	}
	
	protected function _initialize_action_field($_properties) {
		
		$field = $_properties;
		$field['title_message'] = $this->action_title_message;
		$field['description_message'] = $this->action_description_message;
		$field['id'] = $field['id'] . '_action';
		$field['values'] = array(
			'add_and_edit' => $this->action_add_and_edit,
			'add' => $this->action_add,
			'edit' => $this->action_edit
		);
		$field['value'] = 'add_and_edit';
		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($field['id'], 'POST');
		if ($value !== false)
			$this->action_value_formatted = $field['value'] = $value;
		
		$this->_action_field = new SurStudioPluginTranslatorRevolutionLiteRadioField($field);
		
	}
	
	protected function _initialize_language_fields($_properties) {

		$field = $_properties;
		
		$languages = array_merge(array('' => ''), SurStudioPluginTranslatorRevolutionLiteCommon::getLanguages());
		$field['values'] = $languages;

		$field['id'] = $field['id'] . '_from';		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($field['id'], 'POST');
		$this->from_value_formatted = $field['value'] = empty($value) ? '' : $value;
		
		$this->_from_language_field = new SurStudioPluginTranslatorRevolutionLiteSelectField($field);
		
		$field['id'] = $field['id'] . '_to';
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($field['id'], 'POST');
		$this->to_value_formatted = $field['value'] = empty($value) ? '' : $value;
		
		$this->_to_language_field = new SurStudioPluginTranslatorRevolutionLiteSelectField($field);
		
	}
	
	protected function _initialize_resource_field($_properties) {

		$field = $_properties;
		$field['title_message'] = $this->resource_title_message;
		$field['description_message'] = $this->resource_description_message;
		$field['id'] = $field['id'] . '_resource';
		$field['values'] = $this->_get_resource_values();
		
		$value = SurStudioPluginTranslatorRevolutionLiteCommon::getVariable($field['id'], 'POST');
		$this->resource_value_formatted = $field['value'] = empty($value) ? '' : $value;
		
		$this->_resource_field = new SurStudioPluginTranslatorRevolutionLiteSelectField($field);
		
	}
	
	public static function getFilesPaths($_folder, $_globals=false) {
		
		$contents = @scandir($_folder);
		$result = array();
		
		foreach ($contents as $name) {
			$file = $_folder . '/' . $name;
			if (SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($name, '.xml') && !SurStudioPluginTranslatorRevolutionLiteCommon::startsWith($name, 'log_') && @is_file($file) && ($_globals || !SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($name, 'global.xml')))
				$result[] = $file;
		}
		
		return $result;
		
	}
	
	protected function _get_files_scopes() {

		if (!is_null(self::$_files_scopes))
			return self::$_files_scopes;

		$files = self::getFilesPaths($this->folder);
		$result = array();
		
		foreach ($files as $file) {
			
			$contents = SurStudioPluginTranslatorRevolutionLiteFileHandler::read($file);
			
			if (!$contents)
				continue;

			$xml = new DOMDocument();
			$xml->preserveWhiteSpace = false;
			
			if (@!$xml->loadXML($contents))
				continue;
			
			$xpath = new DOMXPath($xml); 
			$scope = $xpath->query("/translations")->item(0)->getAttribute('scope');
			$result[base64_encode($scope)] = $scope;

		}

		asort($result);

		return self::$_files_scopes = $result;
		
	}
	
	protected function _get_resource_values() {
		
		$result = $this->_get_files_scopes();
		
		if (empty($result))
			return array('global' => $this->resource_global_message);
		
		$result = array_merge(
			array('all' => $this->resource_all_message), 
			array('global' => $this->resource_global_message), 
			$this->_get_files_scopes()
		);
		
		return $result;
		
	}
	
	protected function _gen_log_formated($_html_encode=false) {
		
		$result = '
				<tr>
					<th>' . $this->resource_title_message . '</th>
					<th>' . $this->log_added_message . '</th>
					<th>' . $this->log_edited_message . '</th>
					<th>' . $this->log_removed_message . '</th>
					<th>' . $this->log_omitted_message . '</th>
					<th>' . $this->log_created_message . '</th>
				</tr>';
		
		foreach ($this->_log as $key => $value)
			$result .= '
				<tr>
					<td>' . $this->_resource_field->values[$key] . '</td>
					<td>' . $value['added'] . '</td>
					<td>' . $value['edited'] . '</td>
					<td>' . $value['removed'] . '</td>
					<td>' . $value['omitted'] . '</td>
					<td>' . (!empty($value['file_created']) ? $this->log_yes_created_message : $this->log_no_created_message) . '</td>
				</tr>';
		
		$this->log_formatted = $result;
		
	}
	
	protected function _formatted_values($_html_encode=false) {
		
		$this->resource_formatted = $this->_resource_field->output('select.tpl', $_html_encode);
		$this->from_formatted = $this->_from_language_field->output('select_single.tpl', $_html_encode);
		$this->to_formatted = $this->_to_language_field->output('select_single.tpl', $_html_encode);
		$this->action_formatted = $this->_action_field->output($_html_encode);
		$this->test_formatted = $this->_test_field->output($_html_encode);

		$this->_gen_log_formated($_html_encode);

		if (!SurStudioPluginTranslatorRevolutionLiteAdminEvents::isImportingTranslations())
			return;

		if (!array_key_exists('from', $this->_errors))
			$this->from_value_formatted = $this->_from_language_field->values[$this->from_value_formatted];

		if (!array_key_exists('to', $this->_errors))
			$this->to_value_formatted = $this->_to_language_field->values[$this->to_value_formatted];

		if (!array_key_exists('resource', $this->_errors))
			$this->resource_value_formatted = $this->_resource_field->values[$this->resource_value_formatted];

		if (!array_key_exists('action', $this->_errors))
			$this->action_value_formatted = $this->_action_field->values[$this->action_value_formatted];

		if (!array_key_exists('test', $this->_errors))
			$this->test_value_formatted = $this->_test_field->values[$this->test_value_formatted];			

	}
	
	public function output($_template='translations_import.tpl', $_html_encode=false) {

		$this->_formatted_values();

		$result = parent::render(array(
			'type' => 'file',
			'content' => '/admin/' . $_template,
			'meta_tag_rules' => array(
				array(
					'expression' => !empty($this->_resource_field->values),
					'tag' => 'has_resource'
				),
				array(
					'expression' => SurStudioPluginTranslatorRevolutionLiteAdminEvents::isImportingTranslations(),
					'tag' => 'is_importing'
				),
				array(
					'expression' => array_key_exists('csv_file', $this->_errors),
					'tag' => 'csv_file.error'
				),
				array(
					'expression' => array_key_exists('from', $this->_errors),
					'tag' => 'from.error'
				),
				array(
					'expression' => array_key_exists('to', $this->_errors),
					'tag' => 'to.error'
				),
				array(
					'expression' => array_key_exists('resource', $this->_errors),
					'tag' => 'resource.error'
				),
				array(
					'expression' => array_key_exists('action', $this->_errors),
					'tag' => 'action.error'
				),
				array(
					'expression' => array_key_exists('test', $this->_errors),
					'tag' => 'test.error'
				),
				array(
					'expression' => !empty($this->_errors),
					'tag' => 'has_errors'
				),
				array(
					'expression' => $this->_test_field->value == 'true',
					'tag' => 'is_test'
				)
			)
		), $_html_encode);

		return $result;

	}
	
}

class SurStudioPluginTranslatorRevolutionLiteItemCollection {

    protected $_position = 0;

    public $collection;

    public function __construct() {

		$this->_position = 0;

	}

    public function add($_object, $_index=null) {

		if (is_null($_index))
			$this->collection[] = $_object;
		else
			$this->collection[$_index] = $_object;

	}

    public function rewind() {

        $this->_position = 0;

    }

    public function current() {

		$keys = array_keys($this->collection);
        return $this->collection[$keys[$this->_position]];

	}

    public function getFirst() {

		$keys = array_keys($this->collection);
        return $this->collection[$keys[0]];

	}

    public function getLast() {

		$keys = array_keys($this->collection);
        return $this->collection[$keys[count($keys)-1]];

	}

    public function key() {

		return $this->_position;

    }

    public function next() {

        ++$this->_position;

	}

    public function count() {

		return count($this->collection);

	}

    public function valid() {

		$keys = array_keys($this->collection);

		if (!isset($keys[$this->_position]))
			return false;

		return isset($this->collection[$keys[$this->_position]]);

	}

	protected function _add_count_for_render() {

		if (count($this->collection) > 0) {
			$i = 0;
			foreach ($this->collection as $item) {
				$item->collection_item_index = $i;
				$i++;
			}
		}

	}

	public function render($_options, $_html_encode=false) {

		$result = '';
		$partial = array();

		$this->_add_count_for_render();

		if (count($this->collection) > 0) 
			foreach ($this->collection as $item)
				$partial[] = $item->output($_html_encode);

		$object = (object) array(
			'collection' => join("\n", $partial),
			'collection_count' => count($partial)
		);

		foreach ($this as $property => $value)
			if (!is_array($value))
				$object->{$property} = $value;				

		if (!array_key_exists('meta_tag_rules', $_options))
			$_options['meta_tag_rules'] = array();

		$_options['meta_tag_rules'][] = array(
			'expression' => count($partial) == 0,
			'tag' => 'collection.is_empty'
		);

		$result = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($object, $_options, $_html_encode);

		return $result;

	}

}

?>
