<?php
/**
 * Translator Revolution WP Plugin
 * http://goo.gl/4E7Mx
 *
 * LICENSE
 *
 * You need to buy a license if you want use this script.
 * http://codecanyon.net/legal/market
 *
 * @package    Translator Revolution WP Plugin
 * @copyright  Copyright (c) 2015, SurStudio, www.surstudio.net
 * @license    http://codecanyon.net/licenses/standard
 * @version    1.8
 * @date       2015-02-02
 */

class SurStudioPluginTranslatorRevolutionLite {

	protected static $_available;

	public static function initialize() {

		add_action('widgets_init', array('SurStudioPluginTranslatorRevolutionLite', '_initialize_widget'));
		add_action('wp_head', array('SurStudioPluginTranslatorRevolutionLite', '_header'));

		add_action('wp_ajax_nopriv_surstudio_plugin_translator_revolution_lite_get_token', array('SurStudioPluginTranslatorRevolutionLite', '_get_token'));
		add_action('wp_ajax_surstudio_plugin_translator_revolution_lite_get_token', array('SurStudioPluginTranslatorRevolutionLite', '_get_token'));

		add_action('wp_ajax_nopriv_surstudio_plugin_translator_revolution_lite_translate', array('SurStudioPluginTranslatorRevolutionLite', '_translate'));
		add_action('wp_ajax_surstudio_plugin_translator_revolution_lite_translate', array('SurStudioPluginTranslatorRevolutionLite', '_translate'));

	}

	public static function _header() {
	
		$custom_css = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('custom_css');
		SurStudioPluginTranslatorRevolutionLiteCommon::renderCSS($custom_css);
		
	}

	public static function _initialize_widget() {

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('location_widget') == 'true')
			register_widget('SurStudioTranslatorRevolutionUI');
		else {
			add_action('wp_head', array('SurStudioPluginTranslatorRevolutionLite', '_initialize_head'));
			add_action('wp_footer', array('SurStudioPluginTranslatorRevolutionLite', '_initialize_footer'));
		}
		
	}

	public static function _initialize_head() {
		
		if (!self::isAvailable())
			return;
		
		SurStudioPluginTranslatorRevolutionLiteCommon::loadStyle('jquery.translator', SurStudioPluginTranslatorRevolutionLiteConfig::getVersion());
		
	}
	
	public static function _initialize_footer() {

		if (!self::isAvailable())
			return;

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('jquery_auto_load'))
			SurStudioPluginTranslatorRevolutionLiteCommon::loadjQuery(SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('jquery_url'));

		SurStudioPluginTranslatorRevolutionLiteCommon::loadScript('jquery.translator.min.js', SurStudioPluginTranslatorRevolutionLiteConfig::getVersion());

		self::_initialize();

	}

	public static function _translate() {

		require_once SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_PROCEDURES . '/translate.class.php';
		exit;
		
	}
	
	public static function _get_token() {
	
		require_once SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_PROCEDURES . '/token.class.php';
		exit;
		
	}
	
	protected static function _is_cache_writable() {
		
		$flag = SurStudioPluginTranslatorRevolutionLiteConfig::getCacheFlag();
		
		if ($flag)
			return true;
		
		$folder = SurStudioPluginTranslatorRevolutionLiteCommon::isFolderWritable(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);

		if ($folder !== true)
			return false;
			
		$files = SurStudioPluginTranslatorRevolutionLiteCommon::areFolderFilesWritable(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_CACHE);
		
		if ($files !== true)
			return false;
		
		SurStudioPluginTranslatorRevolutionLiteConfig::setCacheFlag(true);
		
		return true;
		
	}
	
	protected static function _api_key_validate() {
		
		$value = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('api_key');
		
		if (empty($value))
			return false;
			
		if (strlen($value) != 56 && strlen($value) != 36)
			return false;
		
		return true;
		
	}
	
	protected static function _client_secret_microsoft_validate() {

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('translation_service') != 'ma')
			return true;
		
		$value = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('microsoft_client_secret');
		
		return !empty($value);
				
	}
	
	protected static function _client_id_microsoft_validate() {

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('translation_service') != 'ma')
			return true;
		
		$value = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('microsoft_client_id');
		
		return !empty($value);
		
	}
	
	protected static function _api_key_google_validate() {
		
		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('translation_service') != 'gt')
			return true;
		
		if (!SurStudioPluginTranslatorRevolutionLiteCommon::isOpenSSLInstalled())
			return false;
		
		$value = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('google_api_key');
		
		return !empty($value);
		
	}
	
	public static function isAvailable($_admin=true) {

		if (defined('WP_ADMIN'))
			return false;

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('test_mode') == 'true' && !current_user_can('administrator'))
			return false;

		if (!self::_api_key_validate() || !self::_api_key_google_validate() || !self::_client_id_microsoft_validate() || !self::_client_secret_microsoft_validate())
			return false;

		if (!self::_is_cache_writable())
			return false;

		global $post; //wordpress post global object
		
		if (!is_object($post))
			return true;
		
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingsValues();
		
		if ($post->post_type == 'page' && array_key_exists('exclude_pages', $settings)) {
		
			$pages = $settings['exclude_pages']['value'];
		
			if (in_array($post->ID, $pages))
				return false;
		
		}

		if ($post->post_type == 'post' && array_key_exists('exclude_posts', $settings)) {
		
			$posts = $settings['exclude_posts']['value'];
		
			if (in_array($post->ID, $posts))
				return false;
		
		}
		
		if ($post->post_type == 'post' && array_key_exists('exclude_categories', $settings)) {
		
			$categories = $settings['exclude_categories']['value'];
		
			$post_categories = wp_get_post_categories($post->ID);

			if (SurStudioPluginTranslatorRevolutionLiteCommon::inArray($categories, $post_categories))
				return false;
		
		}
		
		return true;
		
	}
	
	protected static function _initialize() {
		
		$settings = SurStudioPluginTranslatorRevolutionLiteConfig::getSettingsValues();

		$translator = new SurStudioTranslatorRevolutionLite((object) $settings);

		echo $translator->render(array(
			'type' => 'file',
			'content' => '/main.tpl'
		));
		
	}

}

class SurStudioTranslatorRevolutionUI extends WP_Widget {
	
	public function __construct() {
		
		parent::__construct(SurStudioPluginTranslatorRevolutionLiteConfig::getWidgetName(true), SurStudioPluginTranslatorRevolutionLiteConfig::getWidgetName(), array(
			'classname' => SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('widget_class'), 
			'description' => SurStudioPluginTranslatorRevolutionLiteMessage::get('widget_description')
		));

		if (!SurStudioPluginTranslatorRevolutionLite::isAvailable(false))
			return;
		
		if (is_active_widget(false, false, SurStudioPluginTranslatorRevolutionLiteConfig::getWidgetName(true))) {
		
			add_action('wp_head', array('SurStudioPluginTranslatorRevolutionLite', '_initialize_head'));
			add_action('wp_footer', array('SurStudioPluginTranslatorRevolutionLite', '_initialize_footer'));
		
		}
		
	}
 
	public function form($_instance) {
    
		echo '<div class="surstudio_plugin_translator_revolution_lite_sample_language"></div>';
		echo SurStudioPluginTranslatorRevolutionLiteMessage::get('widget_settings', array(SurStudioPluginTranslatorRevolutionLiteCommon::getAdminPluginUrl()));

		return 'noform';

	}

	public function widget($_arguments, $_instance) {

		if (!SurStudioPluginTranslatorRevolutionLite::isAvailable(false))
			return;

		extract($_arguments, EXTR_SKIP);
 
		echo $before_widget;

		echo '<div id="' . SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('bar_container_id') . '"></div>';
		
		echo $after_widget;
	
	}

}

class SurStudioTranslatorRevolutionLite extends SurStudioPluginTranslatorRevolutionLiteItem {
	
	public $options_formatted;
	
	public function __construct($_properties) {

		$this->_properties = $_properties;
		$this->_gen_options();

	}
	
	public function _prepare_option_value($_id, $_value) {
		
		$value = $_value;
				
		if ($_id == 'custom_html' || SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($_id, '_template'))
			$value = SurStudioPluginTranslatorRevolutionLiteCommon::stripBreakLinesAndTabs($value);

		if (SurStudioPluginTranslatorRevolutionLiteCommon::endsWith($_id, '_selector'))
			$value = trim($value);

		if (SurStudioPluginTranslatorRevolutionLiteValidator::isBool($value))
			$value = $value == 'true' || $value === true;
			
		if (in_array($_id, array('on_before_initialize', 'on_initialize', 'on_start', 'on_complete', 'override', 'custom_languages_names', 'native_languages')))
			$value = array('type' => 'literal', 'value' => $value);

		return SurStudioPluginTranslatorRevolutionLiteFastJSON::encode($value);
		
	}
	
	public function render($_options, $_html_encode=false) {
		
		$result = parent::render($_options, $_html_encode);
		return SurStudioPluginTranslatorRevolutionLiteCommon::stripBreakLinesAndTabs($result);
		
	}
	
	protected function _gen_options() {

		$result = array();
		
		foreach ($this->_properties as $key => $property)
			if (array_key_exists('option_id', $property) && !SurStudioPluginTranslatorRevolutionLiteValidator::isEmpty($property['option_id']))
				$result[$key] = array(
					'option_id' => $property['option_id'],
					'value' => $this->_prepare_option_value($key, $property['value'])
				);

		$result['flags_folder'] = array(
			'option_id' => 'flagsFolder',
			'value' => SurStudioPluginTranslatorRevolutionLiteFastJSON::encode(SURSTUDIO_PLUGIN_TRANSLATOR_REVOLUTION_LITE_IMAGES . '/')
		);

		$result['actions_url'] = array(
			'option_id' => 'actionsUrl',
			'value' => SurStudioPluginTranslatorRevolutionLiteFastJSON::encode(admin_url('admin-ajax.php'))
		);

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('native_languages'))
			$result['native_languages'] = array(
				'option_id' => 'nativeLanguagesNames',
				'value' => SurStudioPluginTranslatorRevolutionLiteFastJSON::encode(SurStudioPluginTranslatorRevolutionLiteCommon::getNativeLanguages(SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('languages')))
			);

		if (SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('location_widget') == 'true') {
			$result['bar_container_id'] = array(
				'option_id' => 'barContainerId',
				'value' => $this->_prepare_option_value('bar_container_id', SurStudioPluginTranslatorRevolutionLiteConfig::getSettingValue('bar_container_id'))
			);			
		}
		else if (array_key_exists('bar_container_id', $result))
			unset($result['bar_container_id']);	

		if (!array_key_exists('location_widget', $result)) { //top
			unset($result['widget_class']);
			unset($result['bar_container_id']);
			unset($result['location_custom']);
			unset($result['custom_location_insert_mode']);
			unset($result['custom_parent']);
		}

/*
		if (array_key_exists('location_widget', $result) && !array_key_exists('location_top', $this->_properties))
		if (array_key_exists('location_custom', $result) && !array_key_exists('location_top', $this->_properties))
			unset($result['location_custom']);*/

		$this->options_formatted = SurStudioPluginTranslatorRevolutionLiteCommon::renderObject($result, array(
			'type' => 'html',
			'content' => "\t{{ option_id }}: {{ value }},\n"
		));

	}
	
}

SurStudioPluginTranslatorRevolutionLite::initialize();

?>
